package com.qmth.eds;

import com.qmth.boot.core.security.service.CustomizeAuthorizationService;
import com.qmth.eds.auth.EdsAuthenticationService;
import com.qmth.eds.base.CustomizedSqlInjector;
import com.qmth.eds.threadPool.MyThreadPool;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.core.task.TaskExecutor;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.annotation.Resource;

@SpringBootApplication(scanBasePackages = "com.qmth.*")
@MapperScan({"com.qmth.eds.mapper"})
//主要就是定义扫描的路径从中找出标识了需要装配的类自动装配到spring的bean容器中,做过web开发的同学一定都有用过@Controller，@Service，@Repository注解，查看其源码你会发现，他们中有一个共同的注解@Component，没错@ComponentScan注解默认就会装配标识了@Controller，@Service，@Repository，@Component注解的类到spring容器中
@EntityScan(basePackages = {"com.qmth.eds.entity"})
//用来扫描和发现指定包及其子包中的Entity定义
@EnableTransactionManagement // spring开启事务支持
@EnableAsync // 开启异步任务
@EnableCaching // 开启缓存注解
public class EdsServiceApplication {

//    public static void disableWarning() {
//        try {
//            Field theUnsafe = Unsafe.class.getDeclaredField("theUnsafe");
//            theUnsafe.setAccessible(true);
//            Unsafe u = (Unsafe) theUnsafe.get(null);
//
//            Class cls = Class.forName("jdk.internal.module.IllegalAccessLogger");
//            Field logger = cls.getDeclaredField("logger");
//            u.putObjectVolatile(cls, u.staticFieldOffset(logger), null);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }

    public static void main(String[] args) {
//        disableWarning();
        SpringApplication.run(EdsServiceApplication.class, args);
    }

    @Resource
    private EdsAuthenticationService edsAuthenticationService;

    @Bean
    public CustomizeAuthorizationService customizeAuthorizationService() {
        return registration -> {
            registration.setDefault(edsAuthenticationService);
        };
    }

    @Primary
    @Bean
    public TaskExecutor primaryTaskExecutor() {
        return new MyThreadPool();
    }

    @Bean
    public CustomizedSqlInjector customizedSqlInjector() {
        return new CustomizedSqlInjector();
    }
}
