package com.qmth.eds.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.dto.ExamSyncTotalDownloadDto;
import com.qmth.eds.contant.SystemConstant;
import com.qmth.eds.entity.BasicMessage;
import com.qmth.eds.enums.ExceptionResultEnum;
import com.qmth.eds.enums.MessageEnum;
import com.qmth.eds.mapper.BasicMessageMapper;
import com.qmth.eds.service.BasicMessageService;
import com.qmth.eds.service.ExamSyncTotalService;
import com.qmth.eds.util.SmsUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2022-05-14
 */
@Service
public class BasicMessageServiceImpl extends ServiceImpl<BasicMessageMapper, BasicMessage> implements BasicMessageService {

    @Resource
    private ExamSyncTotalService examSyncTotalService;

    @Resource
    private SmsUtils smsUtils;

    @Override
    public void sendTeachDataChangeNotice(Long schoolId, Long semesterId, Long examTypeId) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String currentDate = LocalDate.now().format(formatter);
        List<ExamSyncTotalDownloadDto> examSyncTotalDownloadDtos = examSyncTotalService.listDownloadRecordInfos(schoolId, semesterId, examTypeId);
        if (!examSyncTotalDownloadDtos.isEmpty()) {
            for (ExamSyncTotalDownloadDto examSyncTotalDownloadDto : examSyncTotalDownloadDtos) {
                // 短信参数
                Map<String, String> map = new HashMap<>();
                map.put("userName", examSyncTotalDownloadDto.getUserName());
                map.put("schoolName", examSyncTotalDownloadDto.getSchoolName());
                map.put("currentDate", currentDate);
                String variableParams = JSONObject.toJSONString(map);
                String remark = "下载的文件有变动，请重新下载";
                saveMessageSendLog(schoolId, examSyncTotalDownloadDto.getUserId(), examSyncTotalDownloadDto.getUserName(), examSyncTotalDownloadDto.getMobileNumber(), variableParams, MessageEnum.NOTICE_OF_TEACH_DATA_CHANGE, examSyncTotalDownloadDto.getId(), remark);
            }
        }
    }


    /**
     * 保存短信发送日志
     *
     * @param schoolId       学校
     * @param userId         用户ID
     * @param userName       用户名
     * @param mobileNumber   手机号（接收短信）
     * @param variableParams 短信参数
     * @param messageType    日志类型
     * @param businessId     涉及数据ID
     * @param remark         备注
     */
    public void saveMessageSendLog(Long schoolId, Long userId, String userName, String mobileNumber, String variableParams, MessageEnum messageType, Long businessId, String remark) {
        BasicMessage basicMessage = new BasicMessage();
        String templateCode = null;
        try {
            String templateCodeParam = "templateCode";
            // code和content
            Map<String, String> stringMap = smsUtils.getCodeAndContentByEnum(messageType);
            if (!stringMap.containsKey(templateCodeParam) || StringUtils.isBlank(stringMap.get(templateCodeParam))) {
                throw ExceptionResultEnum.ERROR.exception("未配置短信模板Code");
            }
            templateCode = stringMap.get(templateCodeParam);

            // 参数校验
            if (StringUtils.isBlank(userName)) {
                throw ExceptionResultEnum.ERROR.exception("用户名称必填");
            }
            if (StringUtils.isBlank(mobileNumber)) {
                throw ExceptionResultEnum.ERROR.exception("用户手机号必填");
            }

            // 调用阿里云短信平台发送短信
            SendSmsResponse sendSmsResponse = smsUtils.sendSms(mobileNumber, templateCode, variableParams);
            if (sendSmsResponse == null) {
                throw ExceptionResultEnum.ERROR.exception("阿里云短信发送接口调用失败");
            }
            basicMessage.setSendStatus(sendSmsResponse.getCode());
            basicMessage.setSendResult(sendSmsResponse.getMessage());

        } catch (Exception e) {
            basicMessage.setSendStatus("SYSTEM_ERROR");
            basicMessage.setSendResult(e.getMessage());
        } finally {
            basicMessage.setId(SystemConstant.getDbUuid());
            basicMessage.setSchoolId(schoolId);
            basicMessage.setUserId(userId);
            basicMessage.setUserName(userName);
            basicMessage.setMobileNumber(mobileNumber);
            basicMessage.setTemplateCode(templateCode);
            basicMessage.setVariableParams(variableParams);
            basicMessage.setMessageType(messageType.name());
            basicMessage.setBusinessOperate(messageType.getName());
            basicMessage.setBusinessId(businessId);
            basicMessage.setRemark(remark);
            basicMessage.setCreateTime(System.currentTimeMillis());
            this.save(basicMessage);
        }
    }
}
