package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.contant.SystemConstant;
import com.qmth.eds.entity.ExamScheduleTask;
import com.qmth.eds.entity.TBSyncTask;
import com.qmth.eds.enums.ExceptionResultEnum;
import com.qmth.eds.enums.TaskResultEnum;
import com.qmth.eds.enums.TaskStatusEnum;
import com.qmth.eds.mapper.TBSyncTaskMapper;
import com.qmth.eds.service.TBSyncTaskService;
import com.qmth.eds.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 异步任务日志
 */
@Service
public class TBSyncTaskServiceImpl extends ServiceImpl<TBSyncTaskMapper, TBSyncTask> implements TBSyncTaskService {

    @Transactional
    @Override
    public TBSyncTask saveTask(ExamScheduleTask examScheduleTask, boolean isAuto) {
        Long userId = isAuto ? 0L : ServletUtil.getRequestUserId();
        QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBSyncTask::getSchoolId, examScheduleTask.getSchoolId())
                .eq(TBSyncTask::getSemesterId, examScheduleTask.getSemesterId())
                .eq(TBSyncTask::getExamTypeId, examScheduleTask.getExamTypeId());
        TBSyncTask tbSyncTask = this.getOne(queryWrapper);
        if (tbSyncTask != null) {
            if (!TaskStatusEnum.FINISH.equals(tbSyncTask.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("有任务未结束");
            }
            this.removeById(tbSyncTask.getId());
        }
        tbSyncTask = new TBSyncTask(examScheduleTask.getSchoolId(), examScheduleTask.getSemesterId(), examScheduleTask.getExamTypeId(), userId);
        tbSyncTask.setId(SystemConstant.getDbUuid());
        this.save(tbSyncTask);
        return tbSyncTask;
    }

    @Override
    public void updateStatusAndResultById(Long id, TaskStatusEnum status, TaskResultEnum result, String errorMessage) {
        UpdateWrapper<TBSyncTask> tbSyncTaskUpdateWrapper = new UpdateWrapper<>();
        tbSyncTaskUpdateWrapper.lambda().set(TBSyncTask::getStatus, status)
                .set(TBSyncTask::getResult, result)
                .set(TBSyncTask::getErrorMessage, errorMessage)
                .eq(TBSyncTask::getId, id);
        this.update(tbSyncTaskUpdateWrapper);
    }

}
